/*
 * Created on May 10, 2005
 *
 */
package com.sunlabs.archsim.classRegistry;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;

/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 * Registry of available component classes and their names/id's.
 * The name is a shorthand used in XML files that define structures that
 * need to be simulated. 
 * Note: full class names could be used insted af shorthands - for now
 * I opted not to use them for the sake of having more readable input.
 * This decision may need to be revistied. 
 */
public class ClassRegistry {
	
	private static int initialCapacity= 64;
	private static float loadFactor= 0.75f;

	// the only copy of the registry
	public static ClassRegistry instance = new ClassRegistry();

	// registered components
	protected HashMap<String, ComponentInfo> components;
	
	protected ClassRegistry() {
		components= new HashMap<String, ComponentInfo>(initialCapacity, loadFactor);
	} // constructor
	
	public static ClassRegistry getInstance() {
		return instance;
	} // getInstance
	
	public void clear() {
		components= new HashMap<String, ComponentInfo>(initialCapacity, loadFactor);
	} // reset
	
	
	/** load the factory registry form an XML file */
	public void load(String fileName) {
		ClassRegistryLoader loader= new ClassRegistryLoader();
		loader.loadFile(fileName);
	}
	
	/** Register a new class. If the class name 
	 * already has been registered, then return false and make
	 * no registration.
	 * 
	 * @param name class name
	 * @param c the class
	 * @return true if the class was successfully registered
	 */
	public boolean register(String name, ComponentInfo cInfo) {
		boolean contains= components.containsKey(name);
		if (!contains) {
			components.put(name, cInfo);
		}
		return !contains;
	} // Register
	
	/** Return a component factory associated with the name.
	 * 
	 * @param name name for the component factory
	 * @return the component factory with the given name, null if not found
	 */
	public Class getComponentType(String name) {
		return components.get(name).componentType;
	} // get
	
	/**
	 * Get a collection of component information structures
	 * @return colelction of component information structures
	 */
	public Collection<ComponentInfo> getInfoCollection() {
		return components.values();
	}
	
	/**
	 * Get a collection of component names
	 * @return collection of component information
	 */
	public  Collection<String> componentNames() {
		return components.keySet();
	}
	
	/**
	 * Get ComponentInfo for a comoponent with a given name.
	 * @param name component name
	 * @return ComponentInfo object for the component with the given name. null if
	 *         no such component is in the registry.
	 */
	public ComponentInfo getComponentInfo(String name) {
		return components.get(name);
	}
	
	public boolean containsClass(Class c) {
		return components.containsValue(c);
	} 
	
	public int size() {
		return components.size();
	}
	
	public String toString() {
		String result= "Component registry: ";
		
		for (Entry e : components.entrySet()) {
			result+= "\nComponent " + e.getKey() + "\n";
			result+= ((ComponentInfo)e.getValue()).toString();
		}
		
		return result;
	}
	
} // ClassRegistry
