/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Mar 11, 2005
 *
 */
package com.sunlabs.archsim.gasp;

import com.sunlabs.archsim.base.simulation.Command;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.base.simulation.Time;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.globals.GlobalDefaults;

/**
 * @author ib27688
 *
 */
public class MergeAction extends Action {
	
	protected static final String INA= "inA";
	protected static final String ACKA= "ackA";
	protected static final String INB= "inB";
	protected static final String ACKB= "ackB";
	protected static final String OUT= "out";
	protected static final String ACKOUT= "ackOut";	
	
	/** one of the inputs on which arbitration happens */
	protected InputTerminal inA= null;
	protected OutputTerminal ackA= null;

	/** one of the inputs on which arbitration happens */
	protected InputTerminal inB= null;
	protected OutputTerminal ackB= null;

	/* theoutput of this action */
	protected InputTerminal ackOut= null;
	protected OutputTerminal out= null;
	
	/** mapping from inA to out to be defined in a subclass */
	protected Function aToOut= null;

	/** mapping from inB to out to be defined in a subclass */
	protected Function bToOut= null; 
	
	protected boolean aActive;
	protected boolean bActive;
	protected boolean ackOutActive;
	
	/** Delay incurred by arbitration. At the moment we ignore variations */ 
	protected Delay arbitrationDelay;
	
	/** a command used for arbitration */
	protected Command arbitrateCommand;
	
	/** which of the two inputs has the priority */
	char priority;
	
	/** Recored time for outstanding requests. null if not outstanding */
	protected Time requestAOut= null;
	protected Time requestBOut= null;
	
	/** Which of the requests was granted?
	 *  null means none.
	 */
	protected InputTerminal grantedRequest;
	
	/** Function from input A to output
	 * TO BO DEFINED IN A SUBCLASS. 
	 */
	protected Function inAToOut;
	
	/** Function from input A to output
	 * TO BO DEFINED IN A SUBCLASS. 
	 */
	Function inBToOut;	
	
	
	/** Only subclasses can be instantiated */
	protected MergeAction(String name) {
		super(name);
		aActive= false;
		bActive= false;
		ackOutActive= false;
		priority= 'N';
		arbitrateCommand= new ArbitrateCommand();
		Integer arbDelay= globals.intValue(GlobalDefaults.ARBITER_DELAY);
		arbitrationDelay= (arbDelay != null) 
			? new Delay(arbDelay.intValue())
			: GlobalDefaults.ARBITER_DELAY_DEF;

		grantedRequest= null;
		
		fireCommand= new FireCommand();
		notifyStateIsActiveCommand= new StateActiveCommad();
		notifyStateIsInactiveCommand= new StateInactiveCommand();
		
	} // constructor
	
	/**
	 * Attach a terminal. Only terminals with specific names will be
	 * attached.
	 */
	public void attach(Terminal t) {
		String name= t.getBaseName();
		
		if (name== INA && t instanceof InputTerminal) {
				attachInA(t);
		} 
		else if (name== ACKA && t instanceof OutputTerminal) {
			attachAckA(t);
		} else if (name== INB && t instanceof InputTerminal) {
			attachInB(t);
		} 
		else if (name== ACKB && t instanceof OutputTerminal) {
			attachAckB((OutputTerminal)t);
		} else if (name== ACKOUT && t instanceof InputTerminal) {
			attachAckOut(t);
		} 
		else if (name== OUT && t instanceof OutputTerminal) {
			attachOut(t);
		} else {
			director.fatalError("Terminal " + t.getName() + ", id= "
					+ t.getID() + ", has improper name to be attached to"
					+ "MergeAction " + getName() + ", id= " + getID());
		}
	} // attach
	
	
	/**
	 * Command to be invoked when a state becomes active.
	 */
	protected class StateActiveCommad extends Command {
		public void execute(Object param) {
			Terminal src= (Terminal) param;
			if (journalAction) {
				State state= src.getState();
				director.journal("Action " + getAlias()
						+ ", id= " + getID() + " notifed that state "
						+ state.getAlias() + ", id= " + state.getID() 
						+ " is active");						
			}
			if (src == inA) {
				aActive= true;
				requestAOut= director.getTime();				
				// if ackOut input is active as well, ask for a grant
				if (ackOutActive) {
					director.schedule(arbitrateCommand, inA, 
							arbitrationDelay);
				}
			} else if (src == ackA) {
				// do nothing
			} else if (src == inB) {
				bActive= true;
				requestBOut= director.getTime();
				// if ackOut input is active as well, ask for a grant
				if (ackOutActive) {
					director.schedule(arbitrateCommand, inB, 
							arbitrationDelay);
				}				
			} else if (src == ackB) {
				// do nothing
			} else if (src == ackOut) {
				ackOutActive= true;
				if (grantedRequest != null) {
					if (grantedRequest == inA && requestBOut != null) {
						director.schedule(arbitrateCommand, inB, 
								arbitrationDelay);
					}
					else if (grantedRequest == inB && requestAOut != null){ 
						director.schedule(arbitrateCommand, inB, 
								arbitrationDelay);
					}
					// no request is granted at the moment
					grantedRequest= null;
				}
				else {
					// grantedRequest == null, does arbitration
					// need to take place?
					// check for both inputs
					if (aActive) {
						director.schedule(arbitrateCommand, inA, 
								arbitrationDelay);
					}
					if (bActive) {
						director.schedule(arbitrateCommand, inB, 
								arbitrationDelay);
					}					
				}
				
			} else if (src == out) {
				// do nothing
			}
		}
	} // class StateActiveCommand
	
	/** Command to be invoked to perform arbitration.
	 * This command is scheduled by a MergeAction itself. 
	 */
	protected class ArbitrateCommand extends Command {
		public void execute(Object params) {
			InputTerminal t= (InputTerminal)params;
			if (requestAOut != null && requestBOut != null) {
				// race
				if ( (t == inA && requestAOut.compare(requestBOut) < 0)
					|| (t == inB && requestBOut.compare(requestAOut) < 0) ) {
					// the current requester came sooner, therefore it wins
					grant(t);
				}
				else if (requestAOut.compare(requestBOut) == 0)
					// two requests at the same time, further race
					// the winner will be the one with the higer priority
					if (priority == 'N') {
						// the ungranted input will get the priority
						priority = (t== inA)? 'B' : 'A';
						grant(t);
					}
					else if ( t == inA && priority == 'A' ) {
						// switch the priority
						priority= 'B';
						grant(t);
					}
					else  if (t == inB && priority == 'B') {
						// switch the priority
						priority= 'A';
						grant(t);
					}
					// else the current requester will not win 
			}
			else {
				// only one requester, it wins	
				grant(t);
			}
			
		} // execute
		
		/** grant a request */
		private void grant(InputTerminal t) {
			// compute and schedule firing the output
			// set corresponding input states inactive

			// first find out what function to invoke
			Function f;
			if (t == inA) {
				f= inAToOut;
				requestAOut= null;
				grantedRequest= inA;
				if (!aActive || !ackOutActive) {
					director.fatalError("Action " 
							+ MergeAction.this.getAlias() + ", id= "
							+ MergeAction.this.getID() 
							+ ", granted an arbitration request on terminal" 
							+ t.getAlias() + ", id= " + t.getID() 
							+ ", but not all needed inputs are active: " 
							+ "aActive= " + aActive + " ackOutActive = "
							+ ackOutActive);
					return;
				}
			}
			else { // if (t == inB)
				f= inBToOut;
				requestBOut= null;
				grantedRequest= inB;
				if (!bActive || !ackOutActive) {
					director.fatalError("Action " 
							+ MergeAction.this.getAlias() + ", id= "
							+ MergeAction.this.getID() 
							+ ", granted an arbitration request on terminal" 
							+ t.getAlias() + ", id= " + t.getID() 
							+ ", but not all needed inputs are active: " 
							+ "bActive= " + bActive + " ackOutActive = "
							+ ackOutActive);
					return;
				}
			}
			// compute the result and schedule firing
			TerminalValuePair pair= new TerminalValuePair();
			pair.terminal= out;
			pair.value= f.value();
			director.schedule(fireCommand, pair, f.getDelay());				

		}
	} // arbitrateCommand
	
	/** Command invoked to inform a MergeAction 
	 * that a state is inactive */
	protected class StateInactiveCommand extends Command {
		public void execute(Object params) {
			Terminal t= (Terminal)params;
			if (journalAction) {
				State state= t.getState();
				director.journal("Action " + getAlias()
						+ ", id= " + getID() + " notifed that state "
						+ state.getAlias() + ", id= " + state.getID() 
						+ " is INactive");						
			}			
			if (t == inA) {
				aActive= false;
				if (requestAOut != null) {
					director.fatalError( "Arbitration error in "
							+ "action " + getAlias() + ", id= " + getID()
							+ "Terminal " 
							+ t.getAlias() + ", id= " + t.getID() 
							+ " reported inactive state "
							+ t.getState().getAlias() + ", id= "
							+ t.getState().getID()
							+ ", while a request is in arbitration");
				}
			}
			else if (t == inB) {
				bActive= false;
				if (requestBOut != null) {
					director.fatalError( "Arbitration error in "
							+ "action " + getAlias() + ", id= " + getID()
							+ "Terminal " 
							+ t.getAlias() + ", id= " + t.getID() 
							+ " reported inactive state "
							+ t.getState().getAlias() + ", id= "
							+ t.getState().getID()
							+ ", while a request is in arbitration");
				}
			}
			else if (t == ackOut) {
				ackOutActive= false;
			}
		}
	} // StateInactiveCommand
	
	public void attachInA(Terminal t) {
		if (inA != null) {
			terminals.remove(inA);
		}
		terminals.add(t);
		inA= (InputTerminal)t;
		t.attachAction(this);
	} // attachInA

	public void attachAckA(Terminal t) {
		if (ackA != null) {
			terminals.remove(ackA);
		}
		terminals.add(t);
		ackA= (OutputTerminal)t;
		t.attachAction(this);
	} // attachAckA

	
	public void attachInB(Terminal t) {
		if (inB != null) {
			terminals.remove(inB);
		}
		terminals.add(t);
		inB= (InputTerminal)t;
		t.attachAction(this);
	} // attachInB
	
	public void attachAckB(Terminal t) {
		if (ackB != null) {
			terminals.remove(ackB);
		}
		terminals.add(t);
		ackB= (OutputTerminal)t;
		t.attachAction(this);
	} // attachAckB
	
	public void attachOut(Terminal t) {
		if (out != null) {
			terminals.remove(out);
		}
		terminals.add(t);
		out= (OutputTerminal)t;
		t.attachAction(this);
	} // attachOut

	public void attachAckOut(Terminal t) {
		if (ackOut != null) {
			terminals.remove(ackOut);
		}
		terminals.add(t);
		ackOut= (InputTerminal)t;
		t.attachAction(this);
	} // attachAckOut
	
	
	public void setArbitrationDelay(Delay d) {
		arbitrationDelay= (Delay) d.clone();
	}
	
	public Delay getArbitrationDelay() {
		return arbitrationDelay;
	}
	
	
	public boolean selfCheck() {
		boolean OK= true;

		// run a generic action self check
		OK= OK && super.selfCheck();
		
		OK= OK && ( inAToOut != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined function inAToout");	
		}
		
		OK= OK && (inBToOut != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined function inAToout");	
		}		
		
		OK= OK && (inA != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal inA");	
		}				
		OK= OK && (ackA != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal ackA");	
		}				
		OK= OK && (inB != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal inB");	
		}				
		OK= OK && (ackB != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal ackB");	
		}				
		OK= OK && (out != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal out");	
		}				
		OK= OK && (ackOut != null);
		if (!OK) {
			director.error("Action " + getAlias() + ", id= " + getID() 
					+ ", has undefined terminal ackOut");	
		}				
		
		
		return OK;
	} // selfCheck	

	/** Most likely the function that will always be used.
	 * Provided for convenience.
	 * Note that the input is specified by providing either 
	 * character A for inA or character B for inB.
	 */ 
	protected class CopyFunction extends Function {
		private char myInput;
		
		public CopyFunction(char t) {
			super("Copy");
			
			// make t upper case
			if (t == 'a') t= 'A';
			else if (t == 'b') t= 'B';
			
			myInput= t;
			if (t != 'A' || t!= 'B') {
				// wrong input
				director.fatalError("Illegal input terminal indicator"
						+ t + "provided to the copy function."
						+ " It should be either A or B");
			}
		}
		
		public Object value() {
			InputTerminal in;
			if (myInput == 'A') in= inA;
			else if (myInput == 'B') in= inB;
			else {
				director.fatalError("Illegal input terminal indicator"
						+ myInput + "provided to the copy function."
						+ " It should be either A or B");
				return null;
			}
			
			if (in != null) return in.getData();
			else {
				return null;
			}
		} // value
		
	} // CopyFunction
	
} // Arbiter2To1Action
